import logging
import os.path
import pprint

from .common import *


class DebInfo:
    def __init__(self, deb):
        self.deb = deb
        self.depend_list = []
        self.depend_version = {}
        # self.log = log
        self.shell_list= []


    def __file_index(self):
        """
        检查文件列表/var/lib/dpkg/info
        TODO:list文件名，file获取包内容
        """
        listfile = open("/var/lib/dpkg/info" + "/" + self.deb + ".list", "r", encoding='utf-8')
        file = []
        line = listfile.readline().replace('\r', '').replace('\n', '')  # 读取第一行
        while line:
            file.append(line)  # 列表增加
            line = listfile.readline().replace('\r', '').replace('\n', '')  # 读取下一行

        listfile.close()

        file_res = []
        for f in file:
            file_cmd = "file " + '"' + f + '"'
            # self.log.debug("Run " + file_cmd)
            res_cmd = subprocess.run(file_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            file_res.append(res_cmd.stdout.decode("utf-8"))
        # print(res)
        res = []
        for f in file_res:
            info = f.split(": ")
            if len(info) >= 2:
                res.append([info[0], info[1].strip()])
        return res

    def file_type(self):
        file = self.__file_index()
        index = {
            "conf": [],  # 配置文件数量 /etc 下 text 且非 script 文件 和后缀为conf
            "script": [],  # script 文件
            "exec": [],  # shared object+X_OK | executable
            "desktop": [],  # 后缀为.desktop
            "so": [],  # .so + shared object
            "service": [],  # /lib/systemd/system 下非link
            "multi": [],
            "icon": [],
            "link": []
        }

        is_desktop = re.compile(r"\.desktop$")
        is_serv = re.compile(r"\.service$")
        is_script = re.compile(r"script")
        is_elf = re.compile(r"^ELF")
        is_exec = re.compile(r" interpreter ")
        is_so = re.compile(r"\.so|\.so\..*\d$")
        is_etc = re.compile(r"^/etc/")
        is_link = re.compile(r"symbolic link")
        is_xmporsvg = re.compile(r"\.xpm$|\.svg")
        is_media = re.compile(r"\.mp3$|\.wma$\.avi$|\.rm$|\.flv$|\.mpg$|\.mov$|\.mkv$|\.wav$|\.mp4$|\.flv$|\.flac$|\.wmv")
        img_type_list = {'jpg', 'bmp', 'png', 'jpeg',
                         'jfif', 'rgb', 'gif', 'pbm',
                         'ppm', 'pgm', 'rast', 'webp',
                         'svg', 'xpm'}
        for f in file:
            if is_link.search(f[1]): index["link"].append(f[0])
            if is_desktop.search(f[0]): index["desktop"].append(f[0])
            if is_serv.search(f[0]): index["service"].append(f[0])
            if is_script.search(f[1]): index["script"].append(f[0])
            if is_elf.search(f[1]):
                if is_so.search(f[0]):
                    index["so"].append(f[0])
                elif is_exec.search(f[1]):
                    index["exec"].append(f[0])
            if is_etc.search(f[0]) and not is_link.search(f[1]): index["conf"].append(f[0])

            if not is_link.search(f[1]):
                if is_xmporsvg.search(f[0]):
                    index["icon"].append(f[0])
                # if imghdr.what(self.path + "/deb/" + f[0]) in img_type_list:
                #     index["icon"].append(f[0])
                if is_media.search(f[0]):
                    index["multi"].append(f[0])
        return index

    def get_control_info(self):
        info = {
            "pkg_name": "",
            "pkg-version": "",
            "inst_exec": "",
            "uninst_exec": "",
            "depend_num": "0",
            "pkg_arch": ""
        }
        """
        检查包control内容 /var/lib/dpkg/status
        TODO:获取对应包和depend目录
        """
        debian = "/var/lib/dpkg/status"

        last = 0
        get = 0
        control_depend = ""
        with open(debian, "r") as fc:
            for line in fc:
                cont = line.split(":")
                if cont[0].strip() == "Package":
                    info["pkg_name"] = ":".join(cont[1:]).strip()
                    if info["pkg_name"] == self.deb:
                        get = 1
                elif cont[0].strip() == "Version":
                    if get == 1:
                        info["pkg-version"] = ":".join(cont[1:]).strip()
                elif cont[0].strip() == "Depends":
                    if get == 1:
                        else_depends = cont[1].replace('\n', '')
                        if else_depends[-1] == ",":
                            last = 1
                        else:
                            last = 0
                            get = 0
                        control_depend = ":".join(cont[1:]).strip()
                elif last == 1:
                    depends_else = cont[0].replace('\n', '').lstrip()
                    control_depend = control_depend+depends_else
                    if depends_else[-1] == ",":
                        last = 1
                    else:
                        last = 0
                        get = 0
            if control_depend != "":
                self.__get_depend_deb(control_depend)
                info["depend_num"] = len(self.depend_list)
        fc.close()
        return info

    def __get_depend_deb(self, depends: str):
        depend = depends.strip().split(",")
        # self.log.debug(depend)
        for node in depend:
            depend_or = node.split("|")
            tmp = []
            for dep in depend_or:
                # self.log.debug(dep)
                dinfo = dep.strip().replace("(", " ").replace(")", "").split()
                tmp.append(dinfo[0])
                if dinfo[0] not in self.depend_version:
                    self.depend_version[dinfo[0]] = [dinfo[1:]]
                else:
                    self.depend_version[dinfo[0]].append(dinfo[1:])
            self.depend_list.append(tmp)
