
import re

from .common import *
import platform


class ElfDependCheck:
    def __init__(self, elfs: list,arch):
        apt_list = ""
        self.elfs = elfs
        self.un_check_so = []
        self.un_check_deb = []
        self.sopath=[]
        self.arch =arch
        self.check_res=[]

        self.so_info = {}
        # 依赖信息，结构见示例
        # exp1 = {
        #     'libxkbcommon.so.0': {
        #           elf: ["elf1", "elf2"],
        #           runso: '/usr/lib/x86_64-linux-gnu/libxkbcommon.so.0.0.0'
        #           checkso: '---/usr/lib/x86_64-linux-gnu/libxkbcommon.so.0.0.0'
        #     }
        # }
        self.deb_info = {}
        # exp2 = {
        #     "so": ['libxkbcommon.so.0'],
        #     "runso_version": "",
        #     "checkso_version": ""
        # }
        self.so_vers_missing = {}
        self.elf_vers_missing = {}
        #{"libc6.so.0":{"GLIBC_2.28":[elf1 elf2]}}
        self.unknown_so = []
        # 记录寻找包名失败的so库
        # ['libxkbcommon.so.0']

        self.deb_missing = {}
        # 记录目标源寻找失败的包:{"deb1": "原因","deb2": "原因"}

        self.so_missing = {}
        # 记录寻找目标源包内缺少的so库: {"so1": "原因","so2": "原因"}

    def get_depend(self):
        for elf in self.elfs:
            self.get_elf_depend_so(elf)

        so_list = [[so, self.so_info[so]["runso"]] for so in self.so_info]
        # print(so_list)

        unknown_so = self.get_deb_name(so_list)
        if os.path.realpath("/lib") != "/usr/lib":
            return 0

        for ukso in unknown_so:
            if ukso[1].startswith("/lib") and os.path.exists(ukso[1].replace("/lib", "/usr/lib", 1)):
                ukso[1] = ukso[1].replace("/lib", "/usr/lib", 1)
            elif ukso[1].startswith("/usr/lib") and os.path.exists(ukso[1].replace("/usr/lib", "/lib", 1)):
                ukso[1] = ukso[1].replace("/usr/lib", "/lib", 1)

        unknown_so = self.get_deb_name(unknown_so)

        self.unknown_so.extend([so[0] for so in unknown_so])

        return 0

    def get_elf_depend_so(self, elf):
        needso_cmd = "objdump -p " + elf + " |grep NEEDED|awk -F' ' '{print $2}'"
        cmd_result = subprocess.run(needso_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if cmd_result.returncode:
            print(cmd_result.stderr.decode('utf-8'))
            return 1

        needso = cmd_result.stdout.decode("utf-8").split("\n")

        # self.log.debug("Run: ldd " + elf)
        cmd_result = subprocess.run("ldd " + elf, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if cmd_result.returncode:
            # self.log.error("ldd 运行失败：" + cmd_result.stderr.decode('utf-8'))
            # self.log.debug("ldd 运行失败：" + cmd_result.stderr.decode('utf-8'))
            return 1
        out = cmd_result.stdout.decode('utf-8').split("\n")

        for line in out:
            l = line.split(" ")
            if len(l) != 4: continue
            so = os.path.basename(l[2])
            sopath = os.path.realpath(l[2])
            realso = os.path.basename(sopath)
            if not so:
                continue
            if so in self.un_check_so or realso in self.un_check_so:
                continue
            if so not in needso and realso not in needso:
                continue
            if so in self.so_info or realso in self.so_info:
                self.so_info[so]["elf"].append(elf)
            else:
                self.so_info[so] = { "runso": sopath, "elf": [elf]}

    def get_deb_name(self, so_list):
        if len(so_list) == 0:
            return []
        realso = [so[1] for so in so_list]
        dpkg_s_cmd = "dpkg -S " + ' '.join(realso)
        # self.log.debug("Run: " + dpkg_s_cmd)
        cmd_result = subprocess.run(dpkg_s_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        for so in cmd_result.stdout.decode('utf-8').split("\n"):
            # self.log.debug(so)
            so=re.sub('[\u4e00-\u9fa5]','',so)
            if "：" in so:
                # self.log.debug("uos")
                deb = so.split("：")
                deb[0]=re.sub(' ','',deb[0])
            else:
                # self.log.debug("normal")
                deb = so.split(": ")
            
            if ":" in deb[0]:
                deb_no_arch=deb[0].split(":")
                deb[0]=deb_no_arch[0]

                
            # self.log.debug(deb)
            if len(deb) > 1:
                try:
                    p = realso.index(deb[1])
                except ValueError:
                    continue
                if deb[0] in self.deb_info:
                    self.deb_info[deb[0]]["so"].append(so_list[p][0])
                else:
                    self.deb_info.update({deb[0]: {"so": [so_list[p][0]],
                                                   "runso_version": " ",
                                                  }})
                self.so_info[so_list[p][0]]["deb"] = deb[0]

        unknown_so = []
        for so in cmd_result.stderr.decode('utf-8').split("\n"):
            deb = so.split(" ")
            if len(deb) > 2 and deb[2] in realso:
                unknown_so.append(so_list[realso.index(deb[2])])
        return unknown_so

    def source_cache(self, deb):
        apt_down = 'apt-cache madison ' + self.apt_conf + " " + deb
        self.log.debug("Run " + apt_down)
        cmd_result = subprocess.run(apt_down, shell=True, stdout=subprocess.PIPE)
        if not cmd_result.stdout.decode('utf-8'):
            return ""
        res = cmd_result.stdout.decode('utf-8').split("\n")
        return res[0].split("|")[1].strip()

    __miss_deb_so = []

    def find_checkos_debpend(self):

        debs = list(self.deb_info.keys())
        for deb in debs:
            path = "/var/lib/dpkg/info" + "/" + deb + ".list"
            if not os.path.exists(path):
                path = "/var/lib/dpkg/info" + "/" + deb + ":" + self.arch + ".list"
                if not os.path.exists(path):
                    continue
            listfile = open(path, "r", encoding='utf-8')
            line = listfile.readline().replace('\r', '').replace('\n', '')  # 读取第一行
            while line:
                self.check_res.append(line)  # 列表增加
                line = listfile.readline().replace('\r', '').replace('\n', '')  # 读取下一行

            listfile.close()

    def __unzip(self, debpath):
        work_path = os.getenv("PWD")
        depend_file = self.path + "/depend_file"

        if not os.path.exists(debpath):
            print("Unknown error")
            return 1
        if not os.path.exists(depend_file):
            os.makedirs(depend_file)

        os.chdir(self.path)
        # unzip_cmd = f"for deb in $(ls {self.path}/depend); do dpkg -x {self.path}/depend/$deb {depend_file}; done"
        unzip_cmd = "for deb in $(ls {}); do dpkg -x {}/$deb {}; done".format(debpath, debpath, depend_file)
        # self.log.debug("Run: " + unzip_cmd)
        cmd_result = subprocess.run(unzip_cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        if cmd_result.returncode:
            raise Exception("ERROR: 解压安装包失败:" + cmd_result.stderr.decode('utf-8'))

        os.chdir(work_path)
        return 0

    def __get_depend_deb(self, depends: str):
        depend = depends.strip().split(",")
        depend_list = []
        for node in depend:
            depend_or = node.split("|")
            for dep in depend_or:
                dinfo = dep.strip().replace("(", " ").replace(")", "").split()
                depend_list.append(dinfo[0])
        # self.log.debug("获取依赖为: "+ str(depend_list))
        return depend_list

    def is_virtual_pkg(self, so):
        debname = self.so_info[so]["deb"].split(":")[0]
        is_deb = re.compile(r"^{}_\d".format(debname))
        debs = os.listdir(self.path + "/depend")
        deb = ""
        download_path = self.path + "/depend_" + debname
        if not os.path.exists(download_path):
            os.makedirs(download_path)

        for d in debs:
            if is_deb.search(d):
                deb = d
                break
        cmd = "dpkg -I " + self.path + "/depend/" + deb
        # self.log.debug("Run: " + cmd)
        cmd_result = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        info = cmd_result.stdout.decode('utf-8').strip().split("\n")
        deps = []

        for line in info:
            l = line.split(":")
            if l[0].strip() == "Depends":
                deps = self.__get_depend_deb(l[1])
                break
        print("一级依赖库包含：" + str(deps))
        for dep in deps:
            version = self.source_cache(dep)
            if not version:
                raise Exception("未知错误，获取目标源安装包失败: " + dep)

            apt_down = 'cd {}; apt download {} {}={}'.format(download_path, self.apt_conf, dep, version)
            # self.log.debug("Run " + apt_down)
            cmd_result = subprocess.run(apt_down, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            if cmd_result.returncode:
                raise Exception("下载安装包失败:" + cmd_result.stderr.decode('utf-8'))

        self.__unzip(download_path)

        files = []
        for dirpath, dirnames, filenames in os.walk(self.path + "/depend_file"):
            for filename in filenames:
                files.append(os.path.join(dirpath, filename))

        if self.__find_so(so, files):
            # self.log.warning("在{}依赖包中找到动态库{}".format(self.so_info[so]["deb"], so))
            return True

        return False

    def __find_so(self, so, files):
        name = re.escape(so.split(".so")[0] + ".so")
        name2 = re.escape(os.path.basename(self.so_info[so]["runso"]).split(".so")[0] + ".so")
        is_find = False

        is_so = re.compile(r"^{}$|^{}\..*\d$".format(name, name))
        is_so2 = re.compile(r"{}$|{}\..*\d$".format(name2, name2))
        for filepath in files:
            file = os.path.basename(filepath)
            if is_so.search(file) or is_so2.search(file):
                self.so_info[so]["checkso"] = filepath
                is_find = True
        return is_find

    def find_checkos_so(self):
        self.__unzip(self.path + "/depend")

        files = []
        for dirpath, dirnames, filenames in os.walk(self.path + "/depend_file"):
            for filename in filenames:
                files.append(os.path.join(dirpath, filename))

        for so in list(self.so_info.keys()):
            if so in self.un_check_so or so in self.__miss_deb_so:
                continue

            if self.__find_so(so, files):
                continue
            # self.log.warning("未找到依赖库{}，查找deb包{}依赖包是否含有该动态库".format(so, self.so_info[so]["deb"]))
            if not self.is_virtual_pkg(so):
                self.so_missing[so] = "缺失"

    def dep_version_check(self):
        so_info = self.so_info
        elfs = self.elfs
        so_vers = {}
        err_so = {}
        # err_so = {"libc6.so.0":{
        #     "GLIBC_2.28": ["elf1", "elf2"]
        # }}
        for so, info in so_info.items():
            if "checkso" not in info:
                continue
            cmd = "objdump -p " + info["checkso"] + " | grep -P '^\d*\ 0x00' |awk '{print $4}'"
            cmd_result = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            vers = cmd_result.stdout.decode('utf-8').split("\n")[0:-1]

            if not vers:
                vers = []
            so_vers[so] = vers

        # self.log.debug("依赖库版本定义信息:" + str(so_vers))
        is_required = re.compile("^\s{2}required from .*:$")
        is_quote = re.compile("^\s{4}0x\w{8} 0x00")
        for elf in elfs:
            cmd = "objdump -p " + elf
            cmd_result = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            lines = cmd_result.stdout.decode('utf-8').split("\n")[0:-1]

            dep_so = ""
            for line in lines:
                if is_required.search(line):
                    dep_so = line.split()[2][0:-1]
                    # self.log.debug(elf + "引用" + dep_so)
                if is_quote.search(line):
                    ver = line.split()[3]
                    if dep_so not in so_vers:
                        continue
                    if ver in so_vers[dep_so]:
                        # self.log.debug("Find elf：{} dependent version {} in so:{}".format(elf, ver, dep_so))
                        continue
                    # self.log.error("No Find elf：{} dependent version {} in so:{}".format(elf, ver, dep_so))
                    # self.log.debug("No Find elf：{} dependent version {} in so:{}".format(elf, ver, dep_so))
                    if dep_so in err_so:
                        if ver in err_so[dep_so]:
                            err_so[dep_so][ver].append(elf)
                        else:
                            err_so[dep_so][ver] = [elf]
                    else:
                        err_so[dep_so] = {ver:[elf]}
        self.so_vers_missing = err_so

    def elf_version_check(self):
        so_info = self.so_info
        elfs = self.elfs
        elf_vers = {}
        err_so = {}
        # err_so = {"libc6.so.0":{
        #     "GLIBC_2.28": ["elf1", "elf2"]
        # }}
        for elf in elfs:
            cmd = "objdump -p " + elf + " | grep -P '^\d*\ 0x00' |awk '{print $4}'"
            cmd_result = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            vers = cmd_result.stdout.decode('utf-8').split("\n")[0:-1]

            if not vers:
                vers = []
            key = elf.split("/")[-1]
            elf_vers[key] = vers

        is_required = re.compile("^\s{2}required from .*:$")
        is_quote = re.compile("^\s{4}0x\w{8} 0x00")

        for so, info in so_info.items():
            if "checkso" not in info:
                continue
            cmd = "objdump -p " + info["checkso"]
            cmd_result = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            lines = cmd_result.stdout.decode('utf-8').split("\n")[0:-1]
            dep_so = ""
            for line in lines:
                if is_required.search(line):
                    dep_so = line.split()[2][0:-1]
                if is_quote.search(line):
                    ver = line.split()[3]
                    if dep_so not in elf_vers:
                        continue
                    if ver in elf_vers[dep_so]:
                        # self.log.debug("Find elf：{} dependent version {} in so:{}".format(so, ver, dep_so))
                        continue
                    # self.log.error("No Find elf：{} dependent version {} in so:{}".format(so, ver, dep_so))
                    # self.log.debug("No Find elf：{} dependent version {} in so:{}".format(so, ver, dep_so))
                    if dep_so in err_so:
                        if ver in err_so[dep_so]:
                            err_so[dep_so][ver].append(so)
                        else:
                            err_so[dep_so][ver] = [so]
                    else:
                        err_so[dep_so] = {ver:[so]}

        self.elf_vers_missing = err_so

    def check(self):
        print("开始获取程序依赖库")
        self.get_depend()
        print("获取程序依赖信息完成")
        print("开始获取程序依赖库所在依赖包")
        self.find_checkos_debpend()
        print("获取程序依赖库所在安装包完成")
        # self.log.info("开始获取目标系统安装包内程序依赖库")
        # self.find_checkos_so()
        # self.log.info("获取目标系统安装包内程序依赖库完成")
        # self.log.info("开始查询程序版本引用情况")
        # self.dep_version_check()
        # self.log.info("查询程序版本引用情况完成")
        # self.log.info("开始查询版本应用包对系统版本引用影响情况")
        # self.elf_version_check()
        # self.log.info("查询版本应用包对系统版本引用影响情况完成")


class DebDependCheck:
    def __init__(self, depend_list, arch):
        # self.log = log
        self.depend_list = depend_list
        self.arch = arch

    def __check_depend_version(self, dep, versions):
        res = False
        for v1 in versions:
            for drule in self.depend_version[dep]:
                if not drule:
                    res = True
                    break
                v2 = drule[1]
                state = drule[0]

        return res

    def check(self):
        check_res = []

        for deps in self.depend_list:
            path="/var/lib/dpkg/info" + "/" + deps[0] + ".list"
            if not os.path.exists(path):
                path="/var/lib/dpkg/info" + "/" + deps[0] + ":" + self.arch + ".list"
                if not os.path.exists(path):
                    continue
            listfile = open(path, "r", encoding='utf-8')
            line = listfile.readline().replace('\r', '').replace('\n', '')  # 读取第一行
            while line:
                check_res.append(line)  # 列表增加
                line = listfile.readline().replace('\r', '').replace('\n', '')  # 读取下一行

            listfile.close()
        return check_res
